@@ -28,7 +28,7 @@ import (
 
 //export compilerCallback
 func compilerCallback(errorLevel C.int, filename *C.char, linenumber C.int, message *C.char, userData unsafe.Pointer) {
-	c := callbackData.Get(*(*uintptr)(userData)).(*Compiler)
+	c := callbackData.Get(userData).(*Compiler)
 	msg := CompilerMessage{
 		Filename: C.GoString(filename),
 		Line:     int(linenumber),
@@ -49,6 +49,8 @@ type Compiler struct {
 	*compiler
 	Errors   []CompilerMessage
 	Warnings []CompilerMessage
+	// used for include callback
+	callbackData unsafe.Pointer
 }
 
 type compiler struct {
@@ -79,11 +81,19 @@ func (c *compiler) finalize() {
 	runtime.SetFinalizer(c, nil)
 }
 
+func (c *Compiler) setCallbackData(ptr unsafe.Pointer) {
+	if c.callbackData != nil {
+		callbackData.Delete(c.callbackData)
+	}
+	c.callbackData = ptr
+}
+
 // Destroy destroys the YARA data structure representing a compiler.
 // Since a Finalizer for the underlying YR_COMPILER structure is
 // automatically set up on creation, it should not be necessary to
 // explicitly call this method.
 func (c *Compiler) Destroy() {
+	c.setCallbackData(nil)
 	if c.compiler != nil {
 		c.compiler.finalize()
 		c.compiler = nil
@@ -108,7 +118,7 @@ func (c *Compiler) AddFile(file *os.File, namespace string) (err error) {
 	defer C.free(unsafe.Pointer(filename))
 	id := callbackData.Put(c)
 	defer callbackData.Delete(id)
-	C.yr_compiler_set_callback(c.cptr, C.YR_COMPILER_CALLBACK_FUNC(C.compilerCallback), unsafe.Pointer(&id))
+	C.yr_compiler_set_callback(c.cptr, C.YR_COMPILER_CALLBACK_FUNC(C.compilerCallback), id)
 	numErrors := int(C.yr_compiler_add_file(c.cptr, fh, ns, filename))
 	if numErrors > 0 {
 		var buf [1024]C.char
@@ -116,7 +126,6 @@ func (c *Compiler) AddFile(file *os.File, namespace string) (err error) {
 			c.cptr, (*C.char)(unsafe.Pointer(&buf[0])), 1024))
 		err = errors.New(msg)
 	}
-	keepAlive(id)
 	keepAlive(c)
 	return
 }
@@ -133,7 +142,7 @@ func (c *Compiler) AddString(rules string, namespace string) (err error) {
 	defer C.free(unsafe.Pointer(crules))
 	id := callbackData.Put(c)
 	defer callbackData.Delete(id)
-	C.yr_compiler_set_callback(c.cptr, C.YR_COMPILER_CALLBACK_FUNC(C.compilerCallback), unsafe.Pointer(&id))
+	C.yr_compiler_set_callback(c.cptr, C.YR_COMPILER_CALLBACK_FUNC(C.compilerCallback), id)
 	numErrors := int(C.yr_compiler_add_string(c.cptr, crules, ns))
 	if numErrors > 0 {
 		var buf [1024]C.char
@@ -141,7 +150,6 @@ func (c *Compiler) AddString(rules string, namespace string) (err error) {
 			c.cptr, (*C.char)(unsafe.Pointer(&buf[0])), 1024))
 		err = errors.New(msg)
 	}
-	keepAlive(id)
 	keepAlive(c)
 	return
 }
